api_key = ''
api_namespace = 'api'
api_version = "v1"
host = "api.unionbilling.com"
protocol = "https"

# Union Models
from union.models import (
    BaseModel,
    Customer,
    Invoice,
    Vendor,
    Bill,
    Order,
    Item,
    Organization,
    PaymentMethod,
    Payments,
    PurchaseOrder,
    Tax
)

# API Client
from union.client import UnionClient

# Union models
MODEL_MAP = [
        {'model': Customer, 'name': 'customer', 'plural': 'customers'},
        {'model': Invoice, 'name': 'invoice', 'plural': 'invoices'},
        {'model': Vendor, 'name': 'vendor', 'plural': 'vendors'},
        {'model': Bill, 'name': 'bill', 'plural': 'bills'},
        {'model': Order, 'name': 'order', 'plural': 'orders'},
        {'model': Item, 'name': 'item', 'plural': 'items'},
        {'model': Organization, 'name': 'organization', 'plural': 'organizations'},
        {'model': PaymentMethod, 'name': 'payment_method', 'plural': 'payment_methods'},
        {'model': Payments, 'name': 'payments', 'plural': 'payments'},
        {'model': PurchaseOrder, 'name': 'purchase_order', 'plural': 'purchase_orders'},
        {'model': Tax, 'name': 'tax', 'plural': 'taxes'},
]



import json
import requests
import union
from six import text_type


#
# Union Errors
#

class UnionError(Exception):
    pass

class APIError(UnionError):
    pass

class AuthenticationError(UnionError):
    pass

class ValidationError(UnionError):
    pass


#
# Union API Client
#

class UnionClient(object):
    def __init__(self, **kwargs):
        self.content_type =  'application/json'
        self.api_key =       union.api_key
        self.host =          union.host
        self.api_version =   union.api_version
        self.api_namespace = union.api_namespace
        self.protocol =      union.protocol
        self.subpath =       kwargs.get('subpath', '')

    def _headers(self, action=None):
        default = {'Accept': 'application/json',
                   'Authorization': 'Token %s' % (self.api_key,)}

        if action and action.lower() is not 'get':
            return dict(default, **{'Content-Type': 'application/json'})
        return default

    def _model_path_name(self, model):
        if isinstance(model, type):
            class_name = self._look_up_plural_from_model(model)
        else:
            class_name = self._look_up_plural_from_model(model.__class__)
        return '/%s' % class_name

    def _create_url(self, model, **params):
        url = "%s://%s%s" % (self.protocol, self.host, self._model_path_name(model))
        url += self.subpath
        if params:
            if 'id' in list(params):
                id_param = params.pop('id')
                url += '/%s' % id_param
            for k, v in list(params.items()):
                url += '&%s=%s' % (k, v)
        return url

    def _is_valid(self, response):
        return True if (200 <= response.status_code < 300) else False

    def _look_up_plural_from_model(self, model):
        results = [x for x in union.MODEL_MAP if x['model'] == model]
        if results:
            return results[0]['plural']

    def _look_up_model_name(self, name_type, name):
        results = [x for x in union.MODEL_MAP if x[name_type] == name.lower()]
        if results:
            return results[0]['model']

    def _get_model_from_name(self, name):
        if not isinstance(name, text_type):
            return name

        model = self._look_up_model_name('name', name)
        return model or self._look_up_model_name('plural', name)

    def _parse_response(self, response):
        if response.content:
            return self._from_json(json.loads(response.content))

    def _from_json(self, json_data):
        for k, v in list(json_data.items()):
            model = self._get_model_from_name(k)

            if isinstance(v, list):
                return [self._from_json({model: obj}) for obj in v]
            elif isinstance(v, dict) and not isinstance(v, union.BaseModel):
                return model(**v)
            return json_data

    def make_request(self, model, action, url_params={}, post_data=None):
        '''
        Send request to API then validate, parse, and return the response
        '''
        url = self._create_url(model, **url_params)
        headers = self._headers(action)

        try:
            response = requests.request(action, url, headers=headers, data=post_data)
        except Exception as e:
            raise APIError("There was an error communicating with Union: %s" % e)

        if not self._is_valid(response):
            raise ValidationError("The Union response returned an error: %s" % response.content)

        return self._parse_response(response)


from setuptools import setup, find_packages


VERSION = (0, 1, 10)
version = '.'.join(map(str, VERSION))

setup(
    name='union',
    version=version,
    author='Union Inc.',
    author_email='hello@unionbilling.com',
    description='A Python library for accessing the Union Billing API.',
    url='https://github.com/unionbilling/union-python',
    license='MIT',
    keywords='union billing api',
    packages=find_packages(),

    install_requires=[
        'setuptools',
        'requests>=2.19.1',
        'six',
    ],

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],

)



import json
import union
from six import text_type

#
# Abstract Base Model
#

class BaseModel(object):
    def __init__(self, **attrs):
        for k, v in list(attrs.items()):
            self.__setattr__(k, v)

    def __repr__(self):
        return text_type('< union.%s(%s) >' % (type(self).__name__, self))

    def __str__(self):
        return self._to_json

    def __setattr__(self, k, v=None):
        super(BaseModel, self).__setattr__(k, v)

    @property
    def _to_dict(self):
        return dict((k, v) for k, v in list(self.__dict__.items()) if not k.startswith('_'))

    @property
    def _to_json(self):
        return json.dumps(self._to_dict, sort_keys=True, indent=4)

    @classmethod
    def _new_api_client(cls, **kwargs):
        return union.UnionClient(**kwargs)

    @classmethod
    def all(cls):
        '''
        Returns multiple Union objects
        '''
        client = cls._new_api_client()
        return client.make_request(cls, 'get')

    @classmethod
    def filter(cls, **items):
        '''
        Returns multiple Union objects with search params
        '''
        client = cls._new_api_client(subpath='/search')
        items_dict = dict((k, v) for k, v in list(items.items()))
        json_data = json.dumps(items_dict, sort_keys=True, indent=4)
        return client.make_request(cls, 'post', post_data=json_data)

    @classmethod
    def get(cls, id):
        '''
        Look up one Union object
        '''
        client = cls._new_api_client()
        return client.make_request(cls, 'get', url_params={'id': id})

    def save(self):
        '''
        Save an instance of a Union object
        '''
        client = self._new_api_client()
        params = {'id': self.id} if hasattr(self, 'id') else {}
        action =  'patch' if hasattr(self, 'id') else 'post'
        saved_model = client.make_request(self, action, url_params=params, post_data=self._to_json)
        self.__init__(**saved_model._to_dict)

    @classmethod
    def delete(cls, id):
        '''
        Destroy a Union object
        '''
        client = cls._new_api_client()
        return client.make_request(cls, 'delete', url_params={'id': id})

#
# Union Objects
#

class Customer(BaseModel):
    pass

class Invoice(BaseModel):
    pass

class Vendor(BaseModel):
    pass

class Bill(BaseModel):
    pass

class Order(BaseModel):
    pass

class Item(BaseModel):
    pass

class Organization(BaseModel):
    pass

class PaymentMethod(BaseModel):
    pass

class Payments(BaseModel):
    pass

class PurchaseOrder(BaseModel):
    pass

class Tax(BaseModel):
    pass
